@@ -32,7 +32,7 @@ Box_Data_List_Find::Box_Data_List_Find()
   m_HBox.pack_end(m_Button_Find, Gtk::PACK_SHRINK);
   pack_start(m_HBox, Gtk::PACK_SHRINK);
 
-  m_Button_Find.property_can_default() = true; //TODO: Make this a real method in gtkmm?
+  g_object_set(m_Button_Find.gobj(), "can-default", TRUE, (gpointer)NULL); //TODO: Make this a real method in gtkmm?
 
   show_all_children();
 }
@@ -50,14 +50,16 @@ bool Box_Data_List_Find::fill_from_database()
 {
   Bakery::BusyCursor busy_cursor(get_app_window());
 
-  bool result = Box_DB_Table::fill_from_database();
+  const bool result = Box_DB_Table::fill_from_database();
+  if(!result)
+    return false;
 
   //Field Names:
   create_layout();
 
   m_FieldsShown = get_fields_to_show();
 
-  //m_AddDel.add_item(Gnome::Gda::Value("find"));
+  //m_AddDel.add_item(Glib::ValueBase("find"));
 
   return result;
 }
